#!/bin/env python
# -*- coding:utf-8 -*-

import pymongo
import time
import logging
import json
import os
from pdb        import set_trace  as strace
# from traceback  import format_exc as dumpstack
import traceback

pid = os.getpid()

logging.basicConfig(level=logging.INFO, format="[%(asctime)s][pid:%(process)d %(levelname)s] %(message)s", datefmt='%Y-%m-%d %H:%M:%S')
logger = logging.getLogger()

def log(*args, level='info'):
    log_func = getattr(logger, level)
    log_func("{: ^10}: {}".format(traceback.extract_stack()[-2][2], " ".join([str(_) for _ in args])))

class mongodb(object):
    def __init__(self, **kwargs):
        self.host = "mongo.com"
        self.port = 27017
        self.user = None
        self.passwd = None
        self.db   = "data"
        self.table= "test"
        self.conn = None
        self.cur  = None
        self.level = "info"
        self.is_login = False
        self.mechanism = "SCRAM-SHA-1"
        self.timestamp = 0
        [setattr(self, k, v) for k,v in kwargs.items()]
        # self.login()

    def none_log(self):
        self.level = "debug"

    def en_log(self, level='info'):
        self.level = level

    @property
    def dbs(self):
        """ 获取所有数据库名 """
        self.login()
        return self.conn.list_database_names()

    @property
    def tables(self):
        """ 获取数据库的所有表名 """
        self.login()
        return self._db.list_collection_names()

    @property
    def colums(self):
        self.login()
        return self._table.find_one().keys()

    def get_db_tables(self, db):
        self.login()
        return self.conn[db].list_collection_names()

    def not_timeout(self):
        timestamp = time.time()
        ret = self.timestamp <= timestamp <= (self.timestamp + 1800)
        self.timestamp = timestamp
        return ret

    def login(self):
        if self.is_login:
            if self.not_timeout()
                return True
            else:
                log(f"{self}连接超时，强制重连")
        self.conn = pymongo.MongoClient("mongodb://{host}:{port}/".format(**self.__dict__))
        if self.user and self.passwd:
            auth = self.conn["admin"]
            auth.authenticate(self.user, self.passwd, mechanism=self.mechanism)
        self._db = self.conn[self.db]
        self._table = self._db[self.table]
        self.is_login = True
        self.timestamp = time.time()
        return True

    def drop(self):
        log("删除{}".format(str(self)), level=self.level)
        self._table.drop()

    def __repr__(self):
        return "mongodb://{host}:{port}/{db}/{table}".format( **self.__dict__)

    def __del__(self):
        if self.conn: self.conn.close()

    def format(self, data):
        ret = { k: json.dumps(v) if type(v) in [list, dict] else v for k, v in data.items()}
        return ret

    def keys(self, **kwargs):
        self.login()
        data = self._table.find(kwargs, { "_id": 1})
        return [ _["_id"] for _ in data ]

    def __iter__(self):
        self.login()
        return self._table.find({}, no_cursor_timeout = True).batch_size(200)

    # 两种迭代方式，其实我更喜欢下面这种，上面的更优雅
    # def __iter__(self):
    #     self._t_iter = self._table.find({})
    #     return self

    # def __next__(self):
    #     return next(self._t_iter)

    def get(self, limit=False, offset=0, _cols_={}, order=None, desc=False, **kwargs):

        self.login()
        if _cols_:
            data = self._table.find(kwargs, _cols_)
        else:
            data = self._table.find(kwargs)
        if order:
            data = data.sort(order, -1 if desc else 1)
        if limit:
            data = data.limit(limit)
        if offset:
            data = data.skip(offset)
        return [ _ for _ in data ]

    def search(self, limit=False, offset=0, _cols_={}, **kwargs):
        self.login()
        def __(kwargs):
            return { filed :{'$regex': keyword} for filed,keyword in kwargs.items() }

        if not limit:
            # data = self._table.find(kwargs, _cols_).skip(offset)
            data = self._table.find(__(kwargs)).skip(offset)
        else:
            # data = self._table.find(kwargs, _cols_).limit(limit).skip(offset)
            data = self._table.find(__(kwargs)).limit(limit).skip(offset)
        return [ _ for _ in data ]

    def len_search(self, **kwargs):
        self.login()
        return self._table.find(kwargs).count()

    def __len__(self):
        self.login()
        return self._table.count()

    def count(self, **kwargs):
        self.login()
        return self._table.count_documents(kwargs)

    def __delitem__(self, _id):
        log(str(self), "删除", _id, level=self.level)
        self.login()
        return self._table.delete_one({"_id":_id})

    def __setitem__(self, _id, item):
        if not _id in self:
            item["_id"] = _id
            return self + item
        if "_id" in item : del item["_id"]
        data = self.format(item)
        log(str(self), "修改", _id, data, level=self.level)
        self.login()
        self._table.update_one({"_id":_id}, {"$set":item})

    def clearall(self):
        log("清空 {}".format(str(self)))
        self.login()
        return self._table.remove({})

    def __contains__(self, _id):
        return bool(self[_id])

    def get_params(self, params):
        if type(params) is str:
            desc = params[0] == '!'
            key  = params.strip("!")
            return {"order" : key, "desc" : desc}
        if type(params) is dict:
            return params
        return {"order" : "_id", "desc" : params == -1}

    def __getitem__(self, _id):
        self.login()
        if type(_id) is slice:
            assert (_id.stop or 0) >= (_id.start or 0)
            # log("分片操作， [{}:{}:{}]".format(_id.start, _id.stop, _id.step), level=self.level)
            offset = _id.start or 0
            limit  = (_id.stop or 0) - offset
            kwargs = self.get_params(_id.step)
            return self.get(offset=offset, limit=limit, **kwargs)

        data = self._table.find({"_id":_id})
        # strace()
        if data.count() <= 0:
            return None
        return [_ for _ in data][0]

    def __or__(self, conditions):
        self.login()
        data = self._table.find({
                                    "$or": [
                                        {
                                            k:v
                                        } for k,v in conditions.items()
                                    ]
                                })
        return [ _ for _ in data ]

    def __add__(self, item):
        data = self.format(item)
        log(str(self), "新增", data, level=self.level)
        self.login()
        self._table.insert_one(item)
        return self

    def __iadd__(self , item):
        return self + item



import codecs
import os
import sys
try:
    from setuptools import find_packages,setup
except:
    from distutils.core import find_packages,setup

def read(fname):
    return """
# pymongos
简化了一些`pymongo`的操作，像操作`json`一样操作你的数据。
因为日常用增删查改最多，所以简化主要是围绕这三个操作来的。

## 安装
### 先装依赖
```
python -m pip install pymongos
python3 -m pip install pymongos
```

## 连接数据库

### 无密码连接
```
from pymongos import mongodb
table = mongodb(host='127.0.0.1', db="test", table="test")

# 无日志
table = mongodb(host='127.0.0.1', db="test", table="test", level="debug")
```

### 密码连接
```
from pymongos import mongodb
table = mongodb(host='127.0.0.1', db="test", table="test", user="admin", passwd="123456789")
```

## 插入数据
### 不设置_id（MongoDB随机生成）

```
table += {"字段1": "数据1", "字段2": "数据1"}
```
or
```
table + {"字段1": "数据1", "字段2": "数据1"}
```
![](https://raw.githubusercontent.com/77409/e4ting/master/插入-随机ID.jpg)

> 因为分配的ID是随机的，不建议使用

### 设置_id
```
table["设置_id"] = {"字段1": "数据3", "字段2": "数据4"}
table["test"] = {"爱好": "看书", "书名": "鬼吹灯"}
```
![图片传不上来了](https://raw.githubusercontent.com/77409/e4ting/master/插入指定ID.jpg)

## 删除数据
仅支持指定key方式
```
del table["设置_id"]
```

## 修改
```
table["test"] = {"书名": "鬼吹灯和三体"}
table["test"] = {"书名": "《鬼吹灯》《三体》", "爱好": "看书&游泳"}
table["test"] = {"name" : "e4ting"}
```

## 取数据
### 表长度
```
len(table)
```

### _id是否存在
```
if "test" in table:
    print("yes")
```

### 获取所有_id
```
table.keys()
```


### 根据_id取
```
data = table["test"]
print(data)
```

### 取整个表数据
    数据少的时候才可以这么豪横的用
```
table[:]
# 或
table.get()
```

### 遍历表
    表太大的时候，迭代取
```
for _ in table:
    print(_)
```

### 过滤
```
table[::{"name":"e4ting"}]
# 或
table.get(name="e4ting")
# 或 字段名有中文时
table.get(**{"字段1":"数据1"})
```

### 分段
```
table[:10]
# 或
table[1:10]
# 或
table.get(limit=10, offset=1)
```

## 排序
```
# 按 _id 倒序
table[::-1]

# 按 _id 倒序，分段
table[0:10:-1]

# 按 name 正序
table[::"name"]

# 按 name 正序，分段
table[0:10:"name"]

# 按 name 倒序
table[::"!name"]

# 按 name 倒序，分段取
table[0:10:"!name"]
```

### 搜索
```
table.search(name="4ting")
```
### 搜索分段
```
table.search(limit=10, offset=0, **{"字段1" : "数据"})
```

## 其他操作
### 关闭连接
```
del table
```

### 获取所有库名
```
table.dbs
```

### 获取该库下所有collection
```
table.tables
```

### 获取所有字段名
```
table.colums
```

### 关闭日志
```
table.none_log()
```

### 开启日志
```
table.en_log()
```
"""

LONG_DESCRIPTION = read("README.md")

setup(
    name = "pymongos",
    version = "1.2.0"
    ,
    description = "use MongoDB just like json.",
    long_description = LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    keywords = "MongoDB simple dict json",
    author = "e4ting",
    author_email = "e4tingcom@gmail.com",
    url = "https://github.com/77409/pymongos",
    license = "MIT",
    py_modules = ["pymongos", ],
    install_requires=["pymongo",],
    include_package_data=True,
    zip_safe=True,
)
